<?php

class FormRendererHelper
{
    public function renderThemesListForm($themes)
    {
        $result = '';
        foreach ($themes['themes'] as $id => $theme)
        {
            $result .= '<div class="theme-item">'
                    . '<div class="item-title"><h4>'.$theme['themeName'].'</h4></div>'
                    . '<div class="item-view"><img src="'.$theme['thumbnailUrl']. '" alt="'.$theme['themeName'].'"></div>';
            
            $result .= '<div class="item-actions"><a href="' . $theme['openUrl'] .'">Customize with Themler</a></div>';
            $result .= '<div class="item-actions"><a href="' . $theme['settingsUrl'] . '">Theme Settings</a></div>';
            $result .= '</div>';
        }
        $output = <<<EOF
<div class="theme-list">
<div id="theme-editor-list">{$result}</div>
EOF;
        return $output;
    }

    public function renderForms($themeName)
    {
        $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
        $form = array();
        $this->_renderTemplatesForm($form, $themeDir, $themeName);
        $this->_renderSlidersForms($form, $themeDir, $themeName);
        return $form;
    }

    private function _renderTemplatesForm(&$form, $themeDir, $themeName, $index = 0)
    {
        $sourceTemplates = getTemplates();
        $path = $themeDir . '/templates/templates.php';
        if (file_exists($path)) {
            require_once($path);

            $input = array();
            foreach ($sourceTemplates as $type => $caption) {
                $options = array();
                if (isset($templates[$type])) {
                    $min = 0;
                    $minIndex = 0;
                    $i = 0;
                    $template = $templates[$type];
                    $name = getTemplateName($themeName, $type);

                    foreach ($template as $t) {
                        if (isset($t['name']) && ($t['name'] == $type)) {
                            $minIndex = $i;
                        }
                        $options[] = array('id_option' => $type . '_' . $t['id'], 'name' => $t['caption']);
                        $i++;
                    }

                    if (isset($options) && sizeof($options)) {
                        $input[] = array('type' => 'select', 'label' => $caption, 'name' => $name, 'options' => array('query' => $options, 'id' => 'id_option', 'name' => 'name'));
                        $value = Configuration::get($name);
                        $hasEmptyRecord = is_null($value); // has database record with null value
                        $form['fields_value'][$name] = $hasEmptyRecord || !$value ? $options[$minIndex]['id_option'] : $value;
                    }
                }
            }

            $form['fields_form'][$index]['form'] = array(
                'legend' => array(
                    'title' => 'Template settings',
                    'icon' => 'icon-edit'
                ),
                'input' => $input,
            );
        }
    }

    private function _renderSlidersForms(&$form, $themeDir, $themeName, $index = 1)
    {
        $modes = getSliderModes();
        $sliders = getSliders();
        $defaultSettings = getSliderDefaultSettings($themeDir);
        
        foreach ($sliders as $sliderName => $sliderCaption) {
            $input = array();
            foreach ($modes as $modeName => $modeCaption) {
                $options = array(
                    array(
                    'id_option' => '',
                    'name' => '',
                    )
                );
                for ($i=1; $i<=8; $i++)
                {
                    $r = 24 % $i;
                    if ($r) continue;
                    $options[] = array(
                        'id_option' => 24 / $i,
                        'name' => $i,
                    );
                };
                $name = getSliderSettingsName($themeName, $sliderName, $modeName);
                $value = Configuration::get($name);
                if ($modeName === 'itemsInRow') {
                    $dv = 2;
                    $input[] = array('type' => 'text', 'label' => $modeCaption, 'name' => $name, 'class' => ' fixed-width-xl');
                }
                else {
                    $dv = '';
                    $input[] = array('type' => 'select', 'label' => $modeCaption, 'name' => $name, 'options' => array('query' => $options, 'id' => 'id_option', 'name' => 'name'));
                }
                $defaultValue = isset($defaultSettings) && !empty($defaultSettings[$modeName]) ? $defaultSettings[$modeName] : $dv;
                $hasEmptyRecord = is_null($value); // has database record with null value
                $form['fields_value'][$name] = $hasEmptyRecord ? $dv : ($value ? $value : $defaultValue);
            }
            $form['fields_form'][$index]['form'] = array(
                'legend' => array(
                    'title' => $sliderCaption,
                    'icon' => 'icon-edit'
                ),
                'input' => $input,
            );
            $index++;
        }

        $form['fields_form'][$index]['form'] = array(
            'input' => array(
                array(
                    'type' => 'hidden',
                    'name' => 'theme_name',
                    'value' => $themeName
                ),
            ),
            'submit' => array(
                'title' => 'Save',
                'id' => 'submitSliderSettings',
                'icon' => 'process-icon-save'
            )
        );
    }
}