<?php

function getTranslations($themeDir) {
    $includes = $themeDir . '/includes';
    $translations = array();

    foreach (FilesHelper::enumerateFiles($includes, false) as $file) {
        $info = pathinfo($file['name']);
        $filename = $info['filename'];
        $fileExt = isset($info['extension']) && $info['extension'] ? $info['extension'] : '';
        if ($fileExt !== 'lang')
            continue;

        $content = json_decode($file['content'], true);
        if (!is_array($content)) {
            $translations[$filename] = '[DELETED]';
            continue;
        }

        foreach ($content as $source => $langs) {
            $key = $filename . '_' . md5($source);
            foreach ($langs as $lang => $translate) {
                if (!array_key_exists($key, $translations)) 
                    $translations[$key] = array();
                $translations[$key][$lang] = $translate;
            }
        }        
    }

    return $translations;
}

function updateTranslations($themeDir) {
    $newTranslations = getTranslations($themeDir);
    if (empty($newTranslations))
        return;

    $translations = includeLanguageFiles($themeDir);
    foreach ($newTranslations as $key => $newTranslation) {
        if (is_array($newTranslation)) {
            addTranslation($translations, $key, $newTranslation);
        }
        elseif ($newTranslation === '[DELETED]') {
            removeTranslation($translations, $key);
        }
        else {
            throw new Exception('Wrong translation file'.print_r($translations, true));
        }
    }

    if (!empty($translations)) {
        rebuildTranslations($themeDir, $translations);
    }
}

function includeLanguageFiles($themeDir) {
    $langs = array('en', 'fr', 'it', 'de', 'es');
    $translations = array();
    foreach ($langs as $lang) {
        $langPath = $themeDir . '/lang/' . $lang . '.php';
        if (!file_exists($langPath))
            continue;
        require_once($langPath);
        $translations[$lang] = empty($_LANG) ? array() : $_LANG;
    }

    return $translations;
}

function addTranslation(&$translations, $key, $newTranslation) {
    foreach ($newTranslation as $lang => $translate) {
        if (!array_key_exists($key, $translations[$lang])) {
            $translations[$lang][$key] = $translate;
        }
    }
    return;
}

function removeTranslation(&$translations, $key) {
    foreach ($langs as $lang) {
        $sourceKeys = array_keys($translations[$lang]);
        if (empty($sourceKeys))
            continue;
        $keysToRemove = preg_grep('/'.$name.'/', $sourceKeys);
        foreach ($keysToRemove as $removeKey) {
            unset($translations[$lang][$key]);
        }
    }
    return;
}

function rebuildTranslations($themeDir, $translations) {
    // CHECK: don't remove translation for files that doesn't store in theme - liveedit.tpl
    $files = FilesHelper::checkFiles($themeDir, true, false, false, true);

    foreach ($translations as $lang => $translation) {
        $out = <<<EOT
<?php

global \$_LANG;
\$_LANG = array();


EOT;
        if (!empty($translation)) {
            foreach ($translation as $key => $value) {
                if (preg_match("/([^']+?)([^_]+$)/", $key, $matches)) {
                    $fileName = rtrim($matches[1], '_') . '.tpl';
                    // CHECK: don't remove translation for files that doesn't store in theme - liveedit.tpl
                    if (in_array($fileName, $files)) { // file exists
                        $quoteReplaced = preg_replace("/'/", "\'", $value);
                        $out .= <<<EOT
\$_LANG['$key'] = '$quoteReplaced';

EOT;
                    }
                }
            }
        }
        $out .= <<<EOT

return \$_LANG;

EOT;

        FilesHelper::writeFile($themeDir . '/lang/' . $lang.'.php', $out);
    }
}