<?php

require_once(dirname(__FILE__).'/../../DesignerTabSelf.php');

class AdminAjaxController extends ModuleAdminController
{
    private $_required_php_version;
    private $_php_version;
    private $_baseUrl;
    private $_serverUrl;
    private $_templateHelper;
    private $_formRendrerHelper;

    public function __construct()
    {
        $this->bootstrap = true;
        $this->display = 'view';
        $this->meta_title = $this->l('Billion Themler');
        $this->_php_version = phpversion();
        $this->_required_php_version = '5.3.0';
        $this->_baseUrl = Designer::getBaseUrlWithLastSlash();
        $this->_serverUrl = Tools::getCurrentUrlProtocolPrefix().$_SERVER['HTTP_HOST'];
        $this->_adminUrl = $this->_baseUrl . basename(_PS_ADMIN_DIR_) .'/index.php';
        $this->_templateHelper = new TemplateHelper();
        $this->_formRendererHelper = new FormRendererHelper();

        parent::__construct();
    }

    public function initContent()
    {
        parent::initContent();
        theme_check_memory_limit(true);

        if (Tools::getIsset('invalid_urls')) {// base and server urls have to start with the same symbols
            $this->errors[] = $this->l(sprintf('An error has occurred. The Themler start URL %s does not match Shop Domain or SSL Domain %s. This may be due to one of the following reasons: 1. server protocol does not match (HTTP/HTTPS) 2. www prefix is missing in one of the links', $this->_serverUrl.$_SERVER['REQUEST_URI'], $this->_baseUrl));
        }
        if (Tools::getIsset('invalid_php_version')) {
            $this->errors[] = $this->l(sprintf('Unable to open theme. Your server is running php version %s but Themler requires %s or higher. Please upgrade PHP version to continue.', $this->_php_version,  $this->_required_php_version));
        }
        if (Tools::getIsset('no_project')) {
            $this->errors[] = $this->l(sprintf('The theme `%s` cannot be opened because /project/project.json file is missing or corrupted.', $this->_getThemeName()));
        }

        if ($this->ajax)
        {
            try
            {
                $themeName = $this->_getThemeName();
                $this->_checkPermissions($themeName);
                moveInnerPreview($themeName);
                $data = $this->_getThemlerData($themeName);
                echo $this->_buildPage($themeName);
            }
            catch (PermissionsException $e)
            {
                die(buildErrorsPage($e->getMessage()));
            }
        }
        else
        {
            $edit = Tools::getValue('edit', '');
            if (!empty($edit)) {
                $title = $this->l('Slider settings');
                $this->content .= $this->renderThemeSettingsForm();
            }
            else {
                $title = $this->l('List of Themes');
                $this->content .= $this->_formRendererHelper->renderThemesListForm($this->getThemesInfo());
            }
            $this->context->smarty->assign(array(
                'content' => $this->content,
                'title' => $title,
            ));
        }
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitAddconfiguration'))
        {
            $themeName = $this->_getThemeName();
            $modes = getSliderModes();
            $sliders = getSliders();
            foreach ($sliders as $sliderName => $sliderCaption) 
                foreach ($modes as $modeName => $modeCaption)
                {
                    $name = getSliderSettingsName($themeName, $sliderName, $modeName);
                    $value = Tools::getValue($name, '');
                    if (Validate::isInt($value) || empty($value))
                    {
                        $value = empty($value) ? '' : $value;
                        Configuration::updateValue($name, $value);
                    }
                    else
                        $this->errors[] = $this->l('Invalid value for slider "' . $sliderCaption . '" in "' . $modeCaption . '" mode');
                }

            $sourceTemplates = getTemplates();
            foreach ($sourceTemplates as $type => $caption)
            {
                $name = getTemplateName($themeName, $type);
                $value = Tools::getValue($name, '');
                Configuration::updateValue($name, $value);
            }

            if (!count($this->errors)) {
                $this->confirmations[] = $this->l('The settings have been updated.');
            }
        }
        return parent::postProcess();
    }

    public function setMedia()
    {
        parent::setMedia();
        if (!$this->ajax)
            $this->addCSS(__PS_BASE_URI__ . 'modules/designer/css/style.css');
    }

    private function _getThemeName()
    {
        $themeName = Tools::getValue('theme_name', '');
        if ($errors = $this->_checkErrors($themeName))
            die(Tools::jsonEncode($errors));
        return $themeName;
    }

    private function _checkThemlerRequirements($project, $themeName) 
    {
        $errors = array();
        if ((strpos($this->_baseUrl, $this->_serverUrl) === FALSE) || (strpos($this->_baseUrl, $this->_serverUrl) !== 0)) // base and server urls have to start with the same symbols
            $errors[] = 'invalid_urls';
        if ($this->_php_version && version_compare($this->_php_version, $this->_required_php_version, '<'))
            $errors[] = 'invalid_php_version';
        if (!count((array)$project))
            $errors[] = 'no_project';
        if (count($errors)) {
            $header = 'Location: index.php?controller=AdminAjax&token=' . $this->token . '&theme_name=' . $themeName;
            foreach ($errors as $error)
                $header .= sprintf('&%s=true', $error);

            header($header);
        }
    }

    private function _getThemlerData($themeName)
    {
        $project = getProject($themeName);
        $this->_checkThemlerRequirements($project, $themeName);
        $isNew = isset($_GET["create_theme"]) ? $_GET["create_theme"] : 'false'; // for save and publish new theme
        $isActive = (_THEME_NAME_ === $themeName || $isNew === 'true') ? 'true' : 'false';
        $base = $this->_adminUrl . '?themeName='. $themeName . '&ajax=1&controller=AdminAjax&token=' . $this->token . '&action=';
        $templates = $this->_templateHelper->getThemeTemplates($themeName);

        return json_encode(array(
            'themeName' => $themeName,
            'templates' => $templates,
            'canDuplicateTemplatesConstructors' => $this->_templateHelper->getDuplicatedTemplates(),
            'themes' => $this->getThemesInfo(),
            'url' => array('admin' => $this->_adminUrl, 'preview' => $templates['home']),
            'ajax' => $this->_adminUrl,
            'uploadImage' => $base . 'uploadImage',
            'uploadTheme' => $base . 'uploadTheme',
            'unZip' => $base . 'unZip',
            'isActive' => $isActive,
            'isNew' => $isNew,
            'cmsVersion' => getThemeVersions(),
            'cssJsSources' => getThemeCache(getThemeDir($themeName)),
            'md5Hashes' => getThemeHashes(getThemeDir($themeName)),
            'project' => $project,
            'token' => $this->token,
            'maxRequestSize' => getMaxRequestSize()
        ));
    }

    public function ajaxProcessGetManifest()
    {
        try
        {
            header('Content-Type: text/cache-manifest');
            header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
            header("Cache-Control: post-check=0, pre-check=0", false);
            header("Pragma: no-cache");

            die(theme_load_manifest(Tools::getValue('ver')));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }
    
    public function ajaxProcessInitProject()
    {
        try
        {
            theme_check_memory_limit();
            $themeName = Tools::getValue('themeName', '');
            $data = $this->_getThemlerData($themeName);

            header('Content-Type: application/javascript');
            header("Pragma: no-cache");
            die('window.pluginData = ' . $data .';');
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessExport()
    {
        try
        {
            $info = $this->_getExportInfo();
            $designer = $this->getDesignerObj();
            $response = $designer->export($info);
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessSave()
    {
        try
        {
            $info = $this->_getExportInfo();
            $designer = $this->getDesignerObj();
            $response = $designer->save($info);
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessPublish()
    {
        try
        {
            $designer = $this->getDesignerObj();
            $response = $designer->publish();
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessGetTheme()
    {
        try
        {
            $designer      = isset($_REQUEST['data']['id']) ? new DesignerTabSelf($_REQUEST['data']['id']) : $this->getDesignerObj(); // download by id
            $userThemeName = isset($_REQUEST['data']['themeName']) ? $_REQUEST['data']['themeName'] : '';
            $includeEditor = !isset($_REQUEST['data']['includeEditor']) || $_REQUEST['data']['includeEditor'] === 'true';
            $response      = $designer->getTheme($userThemeName, $includeEditor);
            die($response);
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessRequest()
    {
        try
        {
            $designer = $this->getDesignerObj();
            $newName  = isset($_POST['newName']) ? $_POST['newName'] : '';
            $method   = isset($_POST['method']) ? $_POST['method'] : '';
            if (method_exists($designer, $method))
            {
                switch ($method) {
                    case 'update':
                        $response = $designer->$method();
                        break;
                    default:
                        $response = $designer->$method($newName);
                        break;
                }
                die(Tools::jsonEncode($response));
            }
            die(Tools::jsonEncode(array('result' => 'error', 'message' => "Method $method doesn't exist")));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessGetFiles()
    {
        try
        {
            $mask     = isset($_POST['mask']) ? $_POST['mask'] : null;
            $filter   = isset($_POST['filter']) ? $_POST['filter'] : null;
            $designer = $this->getDesignerObj();
            $response = $designer->getFiles($mask, $filter);
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }
    
    public function ajaxProcessSetFiles()
    {
        try
        {
            $designer = $this->getDesignerObj();
            $files = array();

            $response = null;
            $data = isset($_POST['files']) ? $_POST['files'] : false;
            if ($data) {
                $files = json_decode($data, true);
                if (null === $files) {
                    trigger_error('Data is incorrect in setFiles method: "' . $data . '"', E_USER_ERROR);
                }
                $response = array('result' => 'OK');
            }
            else
            {
                $chunk = new Chunk();

                if (!$chunk->save($this->_getChunkInfo())) {
                    header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
                    return array();
                }

                if ($chunk->last())
                {
                    $files = json_decode($chunk->complete(), true);
                    $response = array('result' => 'done');
                }
                else 
                {
                    $response = array('result' => 'processed');
                }
            }

            if ($files && count($files)) {
                $designer->setFiles($files);
            }
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessFsoZip()
    {
        try
        {
            $chunk = new Chunk();

            if (!$chunk->save($this->_getChunkInfo())) {
                header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
                return array();
            }

            if (!$chunk->last()) {
                return array('result' => 'processed');
            }
            
            $data = json_decode($chunk->complete(), true);
            if (!isset($data['fso']))
                throw new Exception('Empty fso');

            $designer = $this->getDesignerObj();
            $response = $designer->zipFso($data);
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessUnZip()
    {
        $designer = $this->getDesignerObj();
        $response = $designer->unZip();
        die(Tools::jsonEncode($response));
    }

    public function ajaxProcessUploadImage()
    {
        $designer = $this->getDesignerObj();
        $response = $designer->uploadFile();
        die(Tools::jsonEncode($response));
    }

    public function ajaxProcessUploadTheme()
    {
        $designer = $this->getDesignerObj();
        $response = $designer->uploadTheme();
        die(Tools::jsonEncode($response));
    }

    public function ajaxProcessClear()
    {
        try
        {
            $designer = $this->getDesignerObj();
            $response = $designer->chunkClear();
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessReloadTemplatesInfo()
    {
        $themeName = Tools::getValue('themeName', '');
        $response = array('templates' => $this->_templateHelper->getThemeTemplates($themeName));
        die(Tools::jsonEncode($response));
    }
    
    public function ajaxProcessReloadThemesInfo()
    {
        $response = $this->getThemesInfo();
        die(Tools::jsonEncode($response));
    }

    private function _getExportInfo()
    {
        $data = isset($_POST['info']) ? $_POST['info'] : false;
        if ($data === false) {
            $info = $this->_getChunkInfo();
        } else {
            $info = json_decode($data, true);
        }
        if (null === $info) {
            trigger_error('Data is incorrect: "' . $data . '"', E_USER_ERROR);
        }
        return $info;
    }

    private function _getChunkInfo()
    {
        return array(
            'id' => isset($_POST['id']) ? $_POST['id'] : '',
            'content' => isset($_POST['content']) ? $_POST['content'] : '',
            'current' => isset($_POST['current']) ? $_POST['current'] : '',
            'total' => isset($_POST['total']) ? $_POST['total'] : '',
            'encode' => !empty($_POST['encode']),
            'blob' => !empty($_POST['blob'])
        );
    }

    private function _buildPage($themeName)
    {
        $version = theme_get_manifest_version($themeName);
        theme_copy_manifest($themeName, $version);
        $manifestVersion = '&ver=' . $version;

        $token = $this->token;
        $adminUrl = $this->_adminUrl;
        $moduleUrl = $this->_baseUrl . 'modules/designer';

        $manifestPath = FilesHelper::normalizePath(getThemeDir($themeName)) . '/export/themler.manifest';
        $manifest = file_exists($manifestPath) ? sprintf('manifest="%s/Export/Manifest.php"', $moduleUrl) : '';
        $version = time();

        $output = <<<EOT
<!DOCTYPE html>
<html manifest="$adminUrl?ajax=1&action=getManifest&controller=AdminAjax&token=$token$manifestVersion">
<head>
<title>Themler</title>
<meta http-equiv="X-UA-Compatible" content="IE=edge" />

<script type="text/javascript" src="$adminUrl?ajax=1&action=initProject&controller=AdminAjax&token=$token&themeName=$themeName"></script>
<script type="text/javascript" src="$moduleUrl/DataProvider.js?version=$version"></script>
<script type="text/javascript" src="$moduleUrl/loader.js?version=$version"></script>

</head>
<body>

</body>
</html>
EOT;
        return $output;
    }
    
    private function getDesignerObj()
    {
        theme_check_memory_limit();
        $themeName = Tools::getValue('themeName', '');
        if ($errors = $this->_checkErrors($themeName)) {
            die(Tools::jsonEncode($errors));
        }

        return new DesignerTabSelf($themeName);
    }

    private function getThemesInfo()
    {
        return Designer::buildThemesData(self::$currentIndex, basename(_PS_ADMIN_DIR_));
    }

    private function renderThemeSettingsForm()
    {
        $result = $this->_formRendererHelper->renderForms($this->_getThemeName());
        $this->fields_value = $result['fields_value'];
        $this->fields_form = array_values($result['fields_form']);
        $this->multiple_fieldsets = true;
        return parent::renderForm();
    }

    private function _exceptionResponse($e)
    {
        return Tools::jsonEncode(array(
                'result' => 'fail',
                'type' => 'permission',
                'error' => $e->getMessage()
               ));
    }

    private function _checkErrors($themeName = '')
    {
        $themeName = empty($themeName) ? Tools::getValue('themeName', null) : $themeName;
        if (is_null($themeName)) {
            return array(
                'result' => 'fail',
                'type'   => 'themeName',
                'title'  => 'Empty theme name',
                'error'  => 'Empty theme name is invalid'
            );
        }

        return null;
    }

    private function _checkPermissions($themeName)
    {
        $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
        $previewDir = $themeDir . _PREVIEW_SUFFIX_;
        $dirs = array(_PS_OVERRIDE_DIR_, _PS_DOWNLOAD_DIR_, _PS_MODULE_DIR_, _PS_UPLOAD_DIR_, $themeDir, $previewDir);
        foreach ($dirs as $dir) {
            $rootOnly = strpos($dir, '/themes/') === FALSE; // check themes subdirectories only
            $check = FilesHelper::checkFiles($dir, true, true, $rootOnly);
        }

        return true;
    }
    
    /* Methods to support Theme installation in Themler */
}